# Funções de ajuda
m_log <- function(message) {
    cat(message, '\n')
}

n_log <- function(name, number, round_coef=2) {
    cat(name, ':', round(number, round_coef), '\n')
}

# Diferença entre duas médias - Exemplo 1
# Com acesso ao vetor de dados
# Como a questão não fornece os vetores, dados foram criados
# artificialmente
set.seed(1924831250)

old_batteries <- rnorm(100, m=190, sd=20)
new_batteries <- rnorm(100, m=200, sd=40)

tb <- data.frame(
    'old batteries' = old_batteries,
    'new batteries' = new_batteries
)

summary(tb)

(var_test <- var.test(new_batteries, old_batteries))

ifelse(var_test$p.value < 0.05, 
       is_var_equal <- FALSE, 
       is_var_equal <- TRUE)

(ans <- t.test(new_batteries, old_batteries, 
               alternative='greater', 
               mu=7, var.equal=is_var_equal))

n_log('t_c', ans$statistic)
n_log('p-value', ans$p.value)
n_log('degrees of freedom', ans$parameter)

# Gráfico de vizualizãção do teste do t crítico
par(mfrow=c(1, 1), pin=c(6, 4))

alpha = 0.05
deg_of_freedom = ans$parameter
t_c = ans$statistic

curve(dt(x, deg_of_freedom), xlim=c(-5, 5), ylab='')

critical_t = qt(alpha, deg_of_freedom, lower.tail=FALSE) 
x_rej = seq(critical_t, 4, len=1e3)
y_rej = dt(x_rej, deg_of_freedom)

polygon(c(critical_t, x_rej), c(0, y_rej),
        col=adjustcolor('red', alpha.f=0.2),
        border='red')

x_acc = seq(-4, critical_t, len=1e3)
y_acc = dt(x_acc, deg_of_freedom)

polygon(c(critical_t, x_acc), c(0, y_acc),
        col=adjustcolor('darkgreen', alpha.f=0.1),
        border='darkgreen')

points(critical_t, 0, pch=18, col='red')
text(critical_t, -0.01, col='red',
     label='t')
points(t_c, 0, pch=18, col='darkgreen')
text(t_c, -0.01, col='darkgreen',
     label='tc')

# Gráfico de vizualização do método do p-valor
par(mfrow=c(1, 1), pin=c(6, 4))

curve(dt(x, deg_of_freedom), xlim=c(-5, 5), ylab='')

x_rej = seq(t_c, 4, len=1e3)
y_rej = dt(x_rej, deg_of_freedom)

polygon(c(t_c, x_rej), c(0, y_rej),
        col=adjustcolor('darkblue', alpha.f=0.2),
        border='darkblue')

x_acc = seq(-4, t_c, len=1e3)
y_acc = dt(x_acc, deg_of_freedom)

polygon(c(t_c, x_acc), c(0, y_acc),
        col=adjustcolor('darkgreen', alpha.f=0.1),
        border='darkgreen')

points(t_c, 0, pch=18, col='darkblue')
text(t_c, -0.01, col='darkblue',
     label='tc')

# Sem acesso ao vetor de dados (na unha)
# Values
s_1 <- 40 # sd(new_batteries)
s_2 <- 20 # sd(old_batteries)
x_bar_1 <- 200 # mean(new_batteries)
x_bar_2 <- 190 # mean(old_batteries)
n_1 <- 100
n_2 <- 100
mu <- 7

# F test
f_c <- s_1^2 / s_2^2
f_p_value <- 2 * pf(f_c, df1=n_1 - 1, df2=n_2 - 2, lower.tail=F)
n_log('f test p-value', f_p_value)

# t test for unequal variance
s_combined <- sqrt(s_1^2 / n_1 + s_2^2 / n_2)
deg_of_freedom <- (s_1^2 / n_1 + s_2^2 / n_2)^2 / ((1 / (n_1 - 1))*(s_1^2 / n_1)^2 + (1 / (n_2 - 1))*(s_2^2 / n_2)^2)

t_c <- ((x_bar_1 - x_bar_2) - mu) / s_combined

t_p_value <- pt(t_c, df=deg_of_freedom, lower.tail=F)
n_log('t test p-value', t_p_value)
n_log('t_c', t_c)

# Diferença entre duas proporções - Exemplo 2
populations <- c(100, 200)
frequencies <- c(0.38 * populations[[1]], 0.51 * populations[[2]])

prop.test(frequencies, populations, correct=FALSE)

# Na unha
# Values
p_1 <- 0.38
p_2 <- 0.51
n_1 <- populations[[1]]
n_2 <- populations[[2]]
pi_0 <- 0

# z test for equality
p_1_2 <- (n_1 * p_1 + n_2 * p_2) / (n_1 + n_2)
sigma_combined <- sqrt(p_1_2 * (1 - p_1_2) * (1 / n_1 + 1 / n_2))

z_c = ((p_1 - p_2) - pi_0) / sigma_combined
n_log('z_c', z_c)

z_p <- 2 * pnorm(z_c)
n_log('z test p-value', z_p, 3)

# Gráfico de vizialização do teste do valor crítico
par(mfrow=c(1, 1), pin=c(6, 4))

alpha <- 0.05
critical_z <- qnorm(alpha / 2, lower.tail=F)

curve(dnorm(x), xlim=c(-5, 5), ylab='')

x_rej_1 = seq(-4, -critical_z, len=1e3)
y_rej_1 = dnorm(x_rej_1)

polygon(c(x_rej_1, -critical_z), c(y_rej_1, 0),
        col=adjustcolor('red', alpha.f=0.2),
        border='red')

x_rej_2 = seq(critical_z, 4, len=1e3)
y_rej_2 = dnorm(x_rej_2)

polygon(c(critical_z, x_rej_2), c(0, y_rej_2),
        col=adjustcolor('red', alpha.f=0.2),
        border='red')

x_acc = seq(-critical_z, critical_z, len=1e3)
y_acc = dnorm(x_acc)

polygon(c(-critical_z, x_acc, critical_z), c(0, y_acc, 0),
        col=adjustcolor('darkgreen', alpha.f=0.1),
        border='darkgreen')

points(z_c, 0, pch=18, col='red')
text(z_c - 0.1, -0.01, col='red',
     label='zc')

points(critical_z, 0, pch=18, col='red')
text(critical_z, -0.01, col='red',
     label='z')

points(-critical_z, 0, pch=18, col='red')
text(-critical_z + 0.1, -0.01, col='red',
     label='-z')

# Gráfico de vizualização do método do p-valor
par(mfrow=c(1, 1), pin=c(6, 4))

curve(dnorm(x), xlim=c(-5, 5), ylab='')

x_rej_1 = seq(-4, z_c, len=1e3)
y_rej_1 = dnorm(x_rej_1)

polygon(c(x_rej_1, z_c), c(y_rej_1, 0),
        col=adjustcolor('darkblue', alpha.f=0.2),
        border='darkblue')

x_rej_2 = seq(-z_c, 4, len=1e3)
y_rej_2 = dnorm(x_rej_2)

polygon(c(x_rej_2, -z_c), c(0, y_rej_2),
        col=adjustcolor('darkblue', alpha.f=0.2),
        border='darkblue')

x_acc = seq(z_c, -z_c, len=1e3)
y_acc = dnorm(x_acc)

polygon(c(z_c, x_acc, -z_c), c(0, y_acc, 0),
        col=adjustcolor('darkgreen', alpha.f=0.1),
        border='darkgreen')

points(z_c, 0, pch=18, col='darkblue')
text(z_c, -0.01, col='darkblue',
     label='-zc')

points(-z_c, 0, pch=18, col='darkblue')
text(-z_c, -0.01, col='darkblue',
     label='zc')
